<?php

namespace DBLumpx\PaymentRucode\Payment;

use XF;
use XF\Entity\PaymentProfile;
use XF\Entity\PurchaseRequest;
use XF\Http\Request;
use XF\Mvc\Controller;
use XF\Payment\AbstractProvider;
use XF\Payment\CallbackState;
use XF\Purchasable\Purchase;

class rucode extends AbstractProvider
{ 
	public function getTitle()
	{
		return 'RUCODE';
	}

	public function getHookUrl()
	{
		return 'https://rucode.cc/process';
	}

	public function initiatePayment(Controller $controller, PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$paymentProfile = $purchase->paymentProfile;
		$purchaser = $purchase->purchaser;
		$apiKey = $paymentProfile->options['apikey'];
		$shop_login = $paymentProfile->options['shop_login'];
		
		$payment_params = [
			'login'  => $paymentProfile->options['shop_login'],
			'order'  => $purchaseRequest->purchase_request_id,
			'curr'    => utf8_strtoupper($purchase->currency),
			'sum'     => $purchase->cost,
			'desc'   => $purchase->title,
			'email'    => $purchaser->email,
			'cf' => $purchaseRequest->request_key,
			'success_url'  => $purchase->returnUrl,
		];
		$payment_params['sign'] = $this->getSign($payment_params, $apiKey);
		$hookUrl = $this->getHookUrl() . '?' . $controller->router()->buildQueryString($payment_params);
		
		return $controller->redirect($hookUrl);
	}
	
	public function setupCallback(\XF\Http\Request $request)
	{
		$state = new CallbackState();
		$state->transactionId = $request->filter('intid', 'str');
		$state->requestKey = $request->filter('cf', 'str');
		$state->subscriberId = $request->filter('email', 'str');
		$state->signature = $request->filter('sign', 'str');
		$state->paymentStatus = $request->filter('result', 'str');
		$state->costAmount = $request->filter('amount', 'unum');
		$state->costCurrency = $request->filter('currency', 'str');
		$state->_POST = $_POST;
		$state->ip = $request->getIp();
		$state->httpCode = 200;
		return $state;
	}

	public function validateTransaction(CallbackState $state)
	{
		if (!$state->transactionId && !$state->requestKey) {
			$state->logType = 'info';
			$state->logMessage = 'No transaction ID or signature. No action to take.';
			return false;
		}
		return parent::validateTransaction($state);
	}

	public function validatePurchasableData(CallbackState $state)
	{
		$paymentProfile = $state->getPaymentProfile();
		$options = $paymentProfile->options;
		if (empty($options['shop_login'])) {
			$state->logType = 'error';
			$state->logMessage = 'Invalid shop_login';
			return false;
		}
		// check sign
		return true;
	}
	
	public function validateCost(CallbackState $state)
	{
		$purchaseRequest = $state->getPurchaseRequest();
		$cost = $purchaseRequest->cost_amount;
		$currency = $purchaseRequest->cost_currency;

		$costValidated = (round($state->costAmount, 2) == round($cost, 2)
			&& ($state->costCurrency == $currency)
		);
		if (!$costValidated) {
			$state->logType = 'error';
			$state->logMessage = 'OK';
			return false;
		}
		return true;
	}

	public function getPaymentResult(CallbackState $state)
	{
		if ($state->paymentStatus == 'success') {
			$state->paymentResult = CallbackState::PAYMENT_RECEIVED;
		}
	}

	public function completeTransaction(CallbackState $state)
	{
		$purchasableHandler = $state->getPurchasableHandler();
		switch ($state->paymentResult) {
			case CallbackState::PAYMENT_RECEIVED:
				$purchasableHandler->completePurchase($state);
				$purchasableHandler->sendPaymentReceipt($state);
				$state->logType = 'info';
				$state->logMessage = 'OK';
				break;

			default:
				$state->logType = 'info';
				$state->logMessage = 'OK';
				break;
		}
	}

	protected function getSign($params, $key)
	{
		$host = $_SERVER['HTTP_HOST'];
		$arr = [
			$host,
			$params['login'],
			(float)$params['sum'],
			$params['curr'],
			$key,
			(int)$params['order']
		];
		$hash = hash('md5', implode(':', $arr));
		return $hash;
	}
	
	public function prepareLogData(CallbackState $state)
	{
		$state->logDetails = $state->_INPUT;
	}
}